#include <libintl.h>
#include <menu.h>
#include <ncurses.h>
#include <string.h>

#define _(x) gettext(x)

void term_init() {
  initscr();
  if (has_colors()) {
    use_default_colors();
    start_color();
  }

  cbreak();
  noecho();
  keypad(stdscr, TRUE);

  init_pair(1, COLOR_GREEN, -1);
  init_pair(2, COLOR_BLUE, -1);
  start_color();
}

int term_ask(char *text, ITEM **items, int sz) {
  attron(COLOR_PAIR(1) | A_BOLD);

  mvprintw(1, 1, "%s", text);
  char line[strlen(text) + 5];
  for (int i = 0; i < strlen(text); i++) {
    line[i]     = '=';
    line[i + 1] = '\0';
  }
  mvprintw(2, 1, "%s", line);

  attroff(COLOR_PAIR(1) | A_BOLD);
  refresh();

  WINDOW *swin = newwin(LINES - 3, COLS - 1, 3, 1);
  keypad(swin, TRUE);

  MENU *menu = new_menu(items);
  set_menu_win(menu, swin);
  set_menu_sub(menu, swin);
  set_menu_mark(menu, ">>> ");
  set_menu_fore(menu, COLOR_PAIR(2));

  post_menu(menu);
  curs_set(0);
  wrefresh(swin);

  bool done = false;
  int  indx = 0, c = 0;

  while ((c = wgetch(swin))) {
    switch (c) {
    case KEY_DOWN:
      if (indx != (sz + 1))
        indx++;
      menu_driver(menu, REQ_DOWN_ITEM);
      break;
    case KEY_UP:
      if (indx != 0)
        indx--;
      menu_driver(menu, REQ_UP_ITEM);
      break;
    case '\n':
      done = true;
      break;
    }

    wrefresh(swin);
    if (done)
      break;
  }

  curs_set(1);
  free_menu(menu);
  return indx;
}

bool term_yn(char *text) {
  char *yn[] = {_("Yes"), _("No"), NULL};
  int   sz   = sizeof(yn) / sizeof(char *);
  ITEM *items[sz + 1];

  for (int i = 0; i < sz; i++)
    items[i] = new_item(yn[i], "");

  bool res = term_ask(_(text), items, sz) == 0;
  for (int i = 0; i < sz; i++)
    free_item(items[i]);
  clear();

  return res;
}

void term_finish() {
  clear();
  endwin();
}
